#!/usr/bin/perl
use Getopt::Long;

#
# generates the job entries under the  nordugrid-info-group-name=jobs


my %config;
GetOptions("dn:s" => \$config{dn},	     	  
	   "queue:s" => \$config{queue}, 
	   "config:s" => \$conf_file, 
	   "valid-to:i" => \$config{ttl},   
	   "help|h" => \$print_help   
	  ); 
 
if ($print_help) { 
    print "\n  
     		script usage: 
		mandatory arguments: --dn
				     --queue
				     --config
				      
		optional arguments:  --valid-to 
		
		this help:	     --help				  		
		\n";
		exit;
}

if (! ($config{dn} and $config{queue} and $conf_file)) {die "A required argument is missing, see --help "};


#
#  default values
#
$config{ttl}            ||= 600;
$config{gm_mount_point}  = "/jobs";
$config{gm_port}         = 2811;
chomp ($config{hostname} = `/bin/hostname -f`);

#
# initialization from the info.conf 
#

my $queue_block;
open (CONFIGFILE, "<$conf_file") or die "Can't open $conf_file configuration file\n";
while (my $line =<CONFIGFILE>) {
   next if $line =~/^#/;   
   $line=~m/^(\w+)\s*=\s*"(.+)"\s*$/;

   ($1 eq "hostname")	  	   and      $config{$1}=$2;
   ($1 eq "pbs_bin_path")	   and      $qstat_command="$2/qstat";
   ($1 eq "controldir") 	   and      $config{$1}=$2;
   ($1 eq "gm_mount_point")	   and      $config{$1}=$2;
   ($1 eq "gm_port")		   and      $config{$1}=$2;	          

   if ($1 eq "queue_name" and $2 eq $config{queue}) {	       
       $queue_block = "true";
   }

   if ($queue_block) {
       ($1 eq "totalcpus")         and     $config{$1}=$2;
       ($1 eq "nodecpu")  	   and     $config{$1}=$2;
       ($1 eq "nodememory")  	   and     $config{$1}=$2;
       ($1 eq "architecture")  	   and     $config{$1}=$2;
       ($1 eq "opsys") 		   and     push @opsys, $2;             
       ($1 eq "scheduling_policy") and     $config{$1}=$2;
       ($line =~ /^$/) and $queue_block = "";
   }	

}	
close CONFIGFILE;


#
# read the queue information for the queue entry from the qstat
#

open QSTATOUTPUT,   "$qstat_command -f -Q $config{queue} |" or die "Error in executing qstat";
while (my $line= <QSTATOUTPUT>) {       
      if ($line =~ m/ = /) {
      	 chomp($line);	 
      	 my ($qstat_var,$qstat_value) = split("=", $line);	
	 $qstat_var =~ s/\s+//g;      	 
      	 $qstat{$qstat_var}=$qstat_value;
      }
}	    
close QSTATOUTPUT;

#
# read the job info from the PBS qstat to a hash of hashes %hoh_pbsjobs
#

open QSTATOUTPUT,  "$qstat_command -f $config{queue} |" or die "Error in executing qstat";
my ($jobid, $job_var, $job_value, $queue_rank);
while (my $line= <QSTATOUTPUT>) { 
       
      next if ($line =~ /^$/); 
      if ($line =~ /^Job Id:\s+(\d+\.\w+)/) {            
	 $jobid=$1;
	 next;	 	 
      }   
      if ($line =~ / = /)  {
          ($job_var,$job_value) = split (/ = /, $line);
	  $job_var =~ s/\s+//g;
	  chop $job_value;
	  if ($job_var eq "job_state" and $job_value eq "Q") {
	      $queue_rank++;
	      $hoh_pbsjobs{$jobid}{"rank"}=$queue_rank;
	  }              
      }
      $hoh_pbsjobs{$jobid}{$job_var}=$job_value;
} 
close QSTATOUTPUT;


#
# read the list of jobs from the jobdir and create the @gridmanager_jobs 
# the @gridmanager_jobs contains the IDs from the job.ID.status
#

opendir JOBDIR,  $config{controldir} or die "Can't access the directory of the jobstatus files";
my @allfiles= readdir JOBDIR;
@allfiles= grep /\.status/, @allfiles;
closedir JOBDIR;
@gridmanager_jobs = map {$_=~m/job\.(.+)\.status/; $_=$1;} @allfiles;


#
# read the gridmanager jobinfo into a hash of hashes %hoh_gmjobs 
# filter out jobs not belonging to this $queue
#

my $controldir = $config{controldir};
foreach $ID (@gridmanager_jobs) { 
    my $gmjob_local=$controldir."/job.".$ID.".local";
    my $gmjob_status=$controldir."/job.".$ID.".status";
    my $gmjob_failed=$controldir."/job.".$ID.".failed";  
    my $gmjob_description=$controldir."/job.".$ID.".description"; 
    my $gmjob_diag=$controldir."/job.".$ID.".diag"; 
    
    open (GMJOB_LOCAL, "<$gmjob_local");
    my @local_allines=<GMJOB_LOCAL>;
    
    # check that this job belongs to this queue  
    my $the_right_queue= grep /^queue=$config{queue}$/, @local_allines;
    if (!$the_right_queue) {
       close GMJOB_LOCAL;
       next;
    }
    
    # parse the content of the job.ID.local into the %hoh_gmjobs hash 
    foreach my $line (@local_allines) {		  
  	$line=~m/^(\w+)=(.+)$/; 		   	
  	$hoh_gmjobs{$ID}{$1}=$2;    
    }	 
    close GMJOB_LOCAL;

    # read the job.ID.status into "status"
    my $job_status_tmp;
    chomp ($job_status_tmp=`cat $gmjob_status 2>/dev/null`); 
    $hoh_gmjobs{$ID}{"status"}= $job_status_tmp;
    if ($hoh_gmjobs{$ID}{"status"} eq "FINISHED") {
      	my @file_stat = stat $gmjob_status;       	  
      	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime (@file_stat[9]);
        my $file_time = sprintf "%4d%02d%02d%02d%02d%02d%1s", $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z";    
      	$hoh_gmjobs{$ID}{"status"} .= " at: $file_time";	
    }     

    # check for job failure, (job.ID.failed )   "errors"
    if (-e $gmjob_failed) {
  	my $failed_tmp=`cat $gmjob_failed 2>/dev/null`;
	$failed_tmp=~s/\n/\. /g;
  	$hoh_gmjobs{$ID}{"errors"}="JOB FAILURE: $failed_tmp";  	
    } 
    
    # do something with the job.ID.description file
    # read the stdin,stdout,stderr 
    my $rsl_string=`cat $gmjob_description 2>/dev/null`;
    chomp $rsl_string;
    if ($rsl_string=~m/"stdin"\s+=\s+"(\S+)"/) {
    	$hoh_gmjobs{$ID}{"stdin"}=$1;
    }
    if ($rsl_string=~m/"stdout"\s+=\s+"(\S+)"/) {
    	$hoh_gmjobs{$ID}{"stdout"}=$1;      
    }
    if ($rsl_string=~m/"stderr"\s+=\s+"(\S+)"/) {
    	$hoh_gmjobs{$ID}{"stderr"}=$1;	
    }
    if ($rsl_string=~m/"count"\s+=\s+"(\S+)"/) {
    	$hoh_gmjobs{$ID}{"count"}=$1;	
    }    
    # the runtimeenvironment is multivalued, needs further processing at printout
    if ($rsl_string=~m/\("runtimeenvironment"\s+=\s+([^\)]+)/i) {
    	$hoh_gmjobs{$ID}{"runtimeenvironment"}=$1;	
    } 
    #read the job.ID.diag file for the cputime info   
    if (-e $gmjob_diag) {
        my $diag_tmp=`cat $gmjob_diag 2>/dev/null`;
	$diag_tmp=~m/nodename=(\S+)/;	
	$hoh_gmjobs{$ID}{"exec_host"}=$1; 		
	($diag_tmp=~m/WallTime=(\S+)\./) ;
    	$hoh_gmjobs{$ID}{"WallTime"}=int($1/60);
	$diag_tmp=~m/KernelTime=(\S+)\./ ;
	$kerneltime=$1;
	$diag_tmp=~m/UserTime=(\S+)\./;
	$usertime=$1;	
	$cputime_tmp=$kerneltime+$usertime;				
	$hoh_gmjobs{$ID}{"CpuTime"}= int($cputime_tmp/60); 		
   } 	 
} 

#
#Count the running and queued GRID jobs (a subset of the PBS jobs)  
#

$gridrunning = 0;
$gridqueued=  0;
foreach my $ID (keys %hoh_gmjobs){
	 next unless  $hoh_gmjobs{$ID}{"status"} eq "INLRMS";
	 $pbsjobid=$hoh_gmjobs{$ID}{"localid"};
	 if ($hoh_pbsjobs{$pbsjobid}{"job_state"} eq "R"){
	     $gridrunning++;
	 }
	 if ($hoh_pbsjobs{$pbsjobid}{"job_state"} eq "Q"){
	     $gridqueued++;
	 }                 
}

#
# The queue entry 
#
print "dn: $config{dn}\n";

# The objectclasses
print "objectclass: Mds\n";
print "objectclass: nordugrid-queue\n";

# The attributes
print "nordugrid-queue-name: $config{queue}\n";

# nordugrid-queue-status
my $gm_running=`ps -C grid-manager  --no-heading`;
my $gridftp_running= `ps -C gridftp-server  --no-heading`;
if (not $gm_running) {
    print "nordugrid-queue-status: inactive, grid-manager is down\n";   
}
elsif  (  !($gridftp_running) )  {
   print "nordugrid-queue-status: inactive, gridftp is down\n";  
}   
elsif ( !( ($qstat{"enabled"} eq " True") and ($qstat{"started"} eq " True"))) {
	print "nordugrid-queue-status: inactive\n";
}
else {
	print "nordugrid-queue-status: active\n";
}

# nordugrid-queue-running
$qstat{"state_count"}=~ m/Running:(\d+)/;
print "nordugrid-queue-running: $1\n";

#nordugrid-queue-gridrunning
print "nordugrid-queue-gridrunning: $gridrunning\n";

# nordugrid-queue-queued
$qstat{"state_count"}=~ m/Queued:(\d+)/;
print "nordugrid-queue-queued: $1\n";

#nordugrid-queue-gridqueued
print "nordugrid-queue-gridqueued: $gridqueued\n";

# nordugrid-queue-maxrunning
$qstat{"max_running"} and print "nordugrid-queue-maxrunning:", $qstat{"max_running"}, "\n";

# nordugrid-queue-maxinqueue
$qstat{"max_queuable"} and print "nordugrid-queue-maxqueuable:", $qstat{"max_queuable"}, "\n";

# nordugrid-queue-maxuserrun
$qstat{"max_user_run"} and print "nordugrid-queue-maxuserrun:", $qstat{"max_user_run"}, "\n";

# nordugrid-queue-maxcputime
if ($qstat{"resources_max.cput"}) {
    @time=split(":",$qstat{"resources_max.cput"});
    $in_minutes=$time[0]*60+$time[1];
    print "nordugrid-queue-maxcputime:", $in_minutes, "\n";
}
# nordugrid-queue-mincput
if ($qstat{"resources_min.cput"}) {
   @time=split(":",$qstat{"resources_min.cput"});
   $in_minutes=$time[0]*60+$time[1]+1;
   print "nordugrid-queue-mincputime:",$in_minutes , "\n";
}

# nordugrid-queue-defaultcputime
if ($qstat{"resources_default.cput"}) {
    @time=split(":",$qstat{"resources_default.cput"});
    $in_minutes=$time[0]*60+$time[1];
    print "nordugrid-queue-defaultcputime:", $in_minutes, "\n";
}

# nordugrid-queue-schedulingpolicy
$config{scheduling_policy} and print "nordugrid-queue-schedulingpolicy: $config{scheduling_policy}\n";

# nordugrid-queue-totalcpus
$config{totalcpus} and print "nordugrid-queue-totalcpus: $config{totalcpus}\n";

# nordugrid-queue-nodecpu
$config{nodecpu} and print "nordugrid-queue-nodecpu: $config{nodecpu}\n";

# nordugrid-queue-nodememory
$config{nodememory} and print "nordugrid-queue-nodememory: $config{nodememory}\n";

# nordugrid-queue-architecture
$config{architecture} and print "nordugrid-queue-architecture: $config{architecture}\n";

#nordugrid-queue-opsys
foreach my $listentry (@opsys) {
   print "nordugrid-queue-opsys: $listentry\n";
}

# Mds-validfrom/to
&mds_valid;

# The nordugrid-info-group=jobs entry
print "dn: nordugrid-info-group-name=jobs, $config{dn}\n"; 
print "objectclass: Mds\n";
print "objectclass: nordugrid-info-group\n";
print "nordugrid-info-group-name: jobs\n";
# Mds-validfrom/to
&mds_valid;


#
# The job entries
#

# jobs are sorted in a descending order with respect to their jobsubmission time
foreach my $ID (sort {$hoh_gmjobs{$b}{"starttime"} <=> $hoh_gmjobs{$a}{"starttime"}} keys %hoh_gmjobs){
     
     my $pbsjobid=$hoh_gmjobs{$ID}{"localid"};
     my $nordugrid_globalid="gsiftp://".$config{hostname}.":".$config{gm_port}.$config{gm_mount_point}."/".$ID; 	
     print "dn: nordugrid-job-globalid=$nordugrid_globalid, nordugrid-info-group-name=jobs, $config{dn}\n";
     print "objectclass: Mds\n";
     print "objectclass: nordugrid-job\n";   
     print "nordugrid-job-globalid: $nordugrid_globalid\n";
     print "nordugrid-job-globalowner: ", $hoh_gmjobs{$ID}{"subject"}, "\n";
     if ( $hoh_gmjobs{$ID}{"jobname"}) {
         print "nordugrid-job-jobname: ", $hoh_gmjobs{$ID}{"jobname"}, "\n";
     }	
     print "nordugrid-job-submissiontime: ",$hoh_gmjobs{$ID}{"starttime"}, "\n";              
     print "nordugrid-job-execcluster: ", $config{hostname}, "\n";
     print "nordugrid-job-execqueue: ",$config{queue}, "\n";
     if ( $hoh_gmjobs{$ID}{"exec_host"}) {
         print "nordugrid-job-executionnodes: ", $hoh_gmjobs{$ID}{exec_host}, "\n";
     }	     
     #nordugrid-job-status
     if ($hoh_gmjobs{$ID}{"status"} eq "INLRMS") {
         if ($hoh_pbsjobs{"$pbsjobid"}{"job_state"}) {   
              print "nordugrid-job-status: INLRMS: ",     
              $hoh_pbsjobs{"$pbsjobid"}{"job_state"}, "\n";
	 }
	 else {      
	      print "nordugrid-job-status: FINISHING\n";     
         }     	      
	 if ($hoh_pbsjobs{$pbsjobid}{"rank"}) {  
              print "nordugrid-job-queuerank: ", $hoh_pbsjobs{$pbsjobid}{"rank"}, "\n"; 
	 }     
         if ($hoh_pbsjobs{$pbsjobid}{"resources_used.cput"}) {
	     $usedcputime_tmp=$hoh_pbsjobs{$pbsjobid}{"resources_used.cput"};
	     $usedcputime_tmp=~/(^\d+):(\d+):/;	   
	     print "nordugrid-job-usedcputime: ",$1*60+$2, "\n"; 
	 }
         if ($hoh_pbsjobs{$pbsjobid}{"resources_used.walltime"}) {
	    $usedwall_tmp=$hoh_pbsjobs{$pbsjobid}{"resources_used.walltime"};	 
	    $usedwall_tmp=~/(^\d+):(\d+):/;	    
	    print "nordugrid-job-usedwalltime: ",$1*60+$2, "\n" ;	   
	} 
	if ( $hoh_pbsjobs{$pbsjobid}{"resources_used.mem"}) {
            print "nordugrid-job-usedmem: ", $hoh_pbsjobs{$pbsjobid}{"resources_used.mem"}-"kb", "\n";
	}
	if ($hoh_pbsjobs{$pbsjobid}{"Resource_List.cput"}) {
            @time=split(":",$hoh_pbsjobs{$pbsjobid}{"Resource_List.cput"});
            $in_minutes=$time[0]*60+$time[1];
            print "nordugrid-job-reqcput: $in_minutes\n";		
	}
	if ($hoh_pbsjobs{$pbsjobid}{"exec_host"}) {            
            print "nordugrid-job-executionnodes: ", $hoh_pbsjobs{$pbsjobid}{"exec_host"}, "\n";		
	}
	if ($hoh_pbsjobs{$pbsjobid}{"comment"}) {	 
	    print "nordugrid-job-lrmscomment:", $hoh_pbsjobs{$pbsjobid}{"comment"}, "\n";    
	}    
     }
     else {
         print "nordugrid-job-status: ", $hoh_gmjobs{$ID}{"status"}, "\n";
	 if ($hoh_gmjobs{$ID}{"WallTime"}) {
             print "nordugrid-job-usedwalltime: ", $hoh_gmjobs{$ID}{"WallTime"}, "\n";
	 }
	 if ($hoh_gmjobs{$ID}{"CpuTime"})  {   
	     print "nordugrid-job-usedcputime: ", $hoh_gmjobs{$ID}{"CpuTime"}, "\n";
	 }    
     }
     if ($hoh_gmjobs{$ID}{"count"}) {
         print "nordugrid-job-cpucount: ", $hoh_gmjobs{$ID}{"count"}, "\n";
     } 
     if ($hoh_gmjobs{$ID}{"errors"}) {
         print "nordugrid-job-errors: ", $hoh_gmjobs{$ID}{"errors"}, "\n";
     }
     if ($hoh_gmjobs{$ID}{"cleanuptime"}) {
         print "nordugrid-job-sessiondirerasetime: ",$hoh_gmjobs{$ID}{"cleanuptime"}, "\n";
     } 
     if ($hoh_gmjobs{$ID}{"stdin"}) {
         print "nordugrid-job-stdin: ",$hoh_gmjobs{$ID}{"stdin"}, "\n";
     }
     if ($hoh_gmjobs{$ID}{"stdout"}) {
         print "nordugrid-job-stdout: ",$hoh_gmjobs{$ID}{"stdout"}, "\n";
     } 
     if ($hoh_gmjobs{$ID}{"stderr"}) {
         print "nordugrid-job-stderr: ",$hoh_gmjobs{$ID}{"stderr"}, "\n";
     }
     if ($hoh_gmjobs{$ID}{"gmlog"}) {
         print "nordugrid-job-gmlog: ",$hoh_gmjobs{$ID}{"gmlog"}, "\n";
     }     
     if (my $tmpruntimestring = $hoh_gmjobs{$ID}{"runtimeenvironment"}) {          
         while( $tmpruntimestring =~ m/"(\S+)"/g ) {
	    print "nordugrid-job-runtimeenvironment: ",$1, "\n";    
	 }                      
     }
     if ($hoh_gmjobs{$ID}{"clientname"}) {
         print "nordugrid-job-submissionui: ",$hoh_gmjobs{$ID}{"clientname"}, "\n";
     }              	 
        
     # Mds-validfrom/to
     &mds_valid;
}

sub mds_valid {
# Mds-validfrom/to
  my $ttl = $config{ttl};
  my $seconds=time;
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($seconds);
  print "Mds-validfrom: ";
  printf "%4d%02d%02d%02d%02d%02d%1s", 
	$year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
	gmtime ($seconds+$ttl);
  print "Mds-validto: ";
  printf "%4d%02d%02d%02d%02d%02d%1s", 
	$year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
  print "\n";
}
