#!/usr/bin/perl
use Getopt::Long;

#
# populates the nordugrid-se MDS entry
#


GetOptions("dn:s" => \$dn,	     	  
	   "valid-to:i" => \$ttl,     
	   "config:s" => \$conf_file, 
	   "help|h" => \$print_help   
	  ); 
 
if ($print_help) { 
    print "\n  
     		script usage: 
		mandatory arguments: --dn
				     --config
				     
		optional arguments:  --valid-to 
		
		this help:	     --help				  		
		\n";
		exit;
}

if (($dn and $conf_file) eq "") {die "A required argument is missing, see --help "};


#
# initialization
#
open (configfile, "<$conf_file") or die "Can't open $conf_file configuration file\n";
while ($line =<configfile>) {
	next if $line =~/^#/;
	next if $line =~/^$/;
	$line=~m/^(\w+)\s*=\s*"(.+)"\s*$/;
	($conf_var,$conf_value) = ($1,$2);
	
	($1 eq "hostname_command")     and  	$hostname_command=$2;
	($1 eq "hostname")     	       and  	$se_hostname=$2;
	($1 eq "se_alias")             and  	$se_alias=$2;
	($1 eq "se_type")	       and	$se_type=$2;
	($1 eq "se_dir")	       and	$se_dir=$2;
	($1 eq "se_mount")	       and	$se_mount=$2;
	($1 eq "se_port")	       and	$se_port=$2;
	($1 eq "gridmap")              and  	$gridmapfile=$2;
	($1 eq "se_gridmap")           and	$se_gridmapfile=$2;
}	


#
#  default values
#
$ttl ||= 600;
$se_port ||= "2811";
$gridmapfile ||= "/etc/grid-security/grid-mapfile";
$se_gridmapfile ||= $gridmapfile;
$se_hostname || chomp ($se_hostname = `/bin/hostname -f`);

#
# read the mappings from the grid-mapfile to a %users hash
#

open mapfile, "<$se_gridmapfile" or die "can't open gridmapfile";

while($line = <mapfile>) {
    chomp($line);  
    if ( ($sn,$unix_user)= $line =~ m/\"([^"]+)\"\s+(\S+)/) {
        $users{$1}=$unix_user; 	
    }  	
}				    
close mapfile;	


# The DN of the entry
print "dn: $dn\n";

# The objectclasses
print "objectclass: Mds\n";
print "objectclass: nordugrid-se\n";

# The attributes
print "nordugrid-se-name: $se_hostname\n";
print "nordugrid-se-baseurl: gsiftp://$se_hostname:$se_port","$se_dir\n";
print "nordugrid-se-aliasname: $se_alias\n";
print "nordugrid-se-type: $se_type\n";


#nordugrid-se-freespace
$freespace=`df $se_mount`;   			     
$freespace =~ /\n\S+\s+\d+\s+\d+\s+(\d+)\s+\d+/;   
$freespace=int $1/1024; 			     
print "nordugrid-se-freespace: $freespace\n";

#nordugrid-se-authuser
foreach $listentry (keys %users) {
   print "nordugrid-se-authuser: $listentry\n";
}

# Mds-validfrom/to
$seconds=time;
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($seconds);
print "Mds-validfrom: ";
printf "%4d%02d%02d%02d%02d%02d%1s", $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($seconds+$ttl);
print "Mds-validto: ";
printf "%4d%02d%02d%02d%02d%02d%1s", $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
exit;
