#!/usr/bin/perl
use Getopt::Long;

#
# generates the user entries under the  nordugrid-info-group-name=users
#

my %config;
GetOptions("dn:s" => \$config{dn},	     	  	        
	   "config:s" => \$conf_file, 
	   "queue:s" => \$config{queue},
	   "valid-to:i" => \$config{ttl},
	   "help|h" => \$print_help   
	  ); 
 
if ($print_help) { 
    print "\n  
     		script usage: 
		mandatory arguments: --dn
				     --config
				     --queue
				     
		optional arguments:  --valid-to 
		
		this help:	     --help				  		
		\n";
		exit;
}

if (! ($config{dn} and $conf_file and $config{queue}) ) {die "A required argument is missing, see --help "};


#
#  default values
#
$config{ttl} ||= 600;
$config{gridmap} = "/etc/grid-security/grid-mapfile";

#
# initialization from the info.conf 
#

my $queue_block;
open (CONFIGFILE, "<$conf_file") or die "Can't open $conf_file configuration file\n";
while (my $line =<CONFIGFILE>) {
   next if $line =~/^#/;   
   $line=~m/^(\w+)\s*=\s*"(.+)"\s*$/;	   

   ($1 eq "gridmap")		    and   $config{$1} = $2;	   
   ($1 eq "sessiondir") 	    and   $config{$1} = $2; 
   ($1 eq "lrmstype")		    and   $config{$1} = $2;  
   ($1 eq "dedicated_node_string")  and   $config{$1} = $2; 
   ($1 eq "pbs_bin_path")	    and   ( $pbsnodes_command="$2/pbsnodes" and
					    $qstat_command="$2/qstat" and 
					    $qmgr_command="$2/qmgr" 
        				  );	     
   ($1 eq "maui_bin_path")	    and   $showbf_command = "$2/showbf" ;
    
   if ($1 eq "queue_name" and $2 eq $config{queue}) {	       
       $queue_block = "true";
   }

   if ($queue_block) {
       ($1 eq "queue_node_string")  and    $config{$1} = $2; 
       ($1 eq "scheduling_policy")  and    $config{$1} = $2;
       ($line =~ /^$/) and $queue_block = "";
   }	   
           
}	
close CONFIGFILE;


#determine LRMS version/flavour in case of PBS"
if ($config{lrmstype} =~ /pbs/i) {
   my $qmgr_string=`$qmgr_command -c "list server"`;
   $qmgr_string =~ /pbs_version = \b(\D+)_(\d\S+)\b/; 
   $config{lrmstype}=$1;
   $config{lrmsversion}=$2;  
}   

# proces the qstat for @acl_users, queue limits 

open QSTATOUTPUT,   "$qstat_command -f -Q $config{queue} |" or die "Error in executing qstat";
while (my $line= <QSTATOUTPUT>) {  
      if ($line =~ /max_running = (\d+)/) {
          $max_running = $1;
      }
      elsif ($line =~ /max_user_run = (\d+)/) {
          $max_userrun= $1;
      }
       elsif ($line =~ /resources_max.cput = (\w+)/) {
           my @time=split(":",$1);
           $max_cputime=":".($time[0]*60+$time[1]);       	  
      }    
      elsif ($line =~ /acl_user_enable = (\w+)/) {
          $acl_enable=$1;	 
      }
      elsif ( ($acl_enable eq "True") and ($line =~ /acl_users\s+=\s+([\w,]+)/) ) {      
      @acl_users = split (/\,/, $1); 
      }
      else { next;
      }
}	    
close QSTATOUTPUT;

#
# count the number of running jobs & the jobs per unix users in the queue  
# and the number of queueing jobs per users
#

open QSTATOUTPUT,   "$qstat_command -a $config{queue} |" or die "Error in executing qstat";
while (my $line= <QSTATOUTPUT>) {  
      if ($line =~ /^\d\S+\s+(\w+)/) {            
	  my $temp=$1;	          	 
	  if ($line =~ /\sQ\s/) {
	     $user_jobs_queued{$temp}++;	    
	  }   
	  if ($line =~ /\sR\s/) {
	     $user_jobs_running{$temp}++;  
	     $totalrunning_in_the_queue++;
	  }  
      }
}
close QSTATOUTPUT;

#processing the pbsnodes output by using a hash of hashes %hoh_pbsnodes
my %hoh_pbsnodes;
open PBSNODESOUT,  "$pbsnodes_command -a |" or die "Error in executing pbsnodes";
while (my $line= <PBSNODESOUT>) { 
       
      if ($line =~ /^$/) {next}; 
      if ($line =~ /^(\w+)/) {            
	 $nodeid=$1;
	 next;	 	 
      }
      if ($line =~ / = /)  {
          ($node_var,$node_value) = split (/ = /, $line);
	  $node_var =~ s/\s+//g;
	  chop $node_value;	  	  
      }           
      $hoh_pbsnodes{$nodeid}{$node_var}=$node_value;     
} 
close PBSNODESOUT;


# calculate the totalcpus & number of used cpus
# taking into account the minor output difference of OpenPBS and PBSPRO
# skipping the nondedicated nodes in case the $dedicated_node_string is defined
$totalcpus=0;
$usedcpus=0;
my $npstring;
if ( lc($config{lrmstype}) eq "openpbs"){
      $npstring="np" 
}
elsif (lc($config{lrmstype}) eq "pbspro"){
     $npstring="pcpus"
}
else  {die "The given flavour of the PBS is not supported"};  


foreach my $node (keys %hoh_pbsnodes){
      if ($config{dedicated_node_string}){
         next unless ($hoh_pbsnodes{$node}{"properties"} =~ m/$config{dedicated_node_string}/)	  	        	  
      }
      if ($config{queue_node_string}){
          next unless ($hoh_pbsnodes{$node}{"properties"} =~ m/$config{queue_node_string}/) 	  	       
      }      
      my $nodestate=$hoh_pbsnodes{$node}{"state"};      
      if ($nodestate=~/down/ or $nodestate=~/offline/ or $nodestate=~/[^-]busy/) {next};      
      $totalcpus += $hoh_pbsnodes{$node}{$npstring};      
      if ($hoh_pbsnodes{$node}{"jobs"}){
        $usedcpus++;
        my @comma = ($hoh_pbsnodes{$node}{"jobs"}=~ /,/g);
	$usedcpus+=@comma;
      } 
}      


#
# read the mappings from the grid-mapfile to a %users hash, the SN is the key
#

open MAPFILE, "<$config{gridmap}" or die "can't open gridmapfile";
while(my $line = <MAPFILE>) {
    chomp($line);  
    if ($line =~ m/\"([^"]+)\"\s+(\S+)/) {
        $users{$1}=$2; 	
    }
}
close MAPFILE;

#
# in case of a Maui scheduler fill a hash (%unix_mappings) with the unix usernames as keys
# and freecpu info as value
#    

if (lc($config{scheduling_policy}) eq "maui") {
   for ( values(%users)) {
     $unix_mappings{$_}="";   
   } 
#commented out for the temporary solution below    
#    for ( keys %unix_mappings) {
#      open showbfoutput,   "$showbf_command -u  $_ |" or die "Error in executing showbf";
#      while ($line= <showbfoutput>) {  
#    
#      	if ($line =~ /no procs available/) {
#      	  $unix_mappings{$_}= " 0";
#      	# print "no procs\n";
#      	  last; 
#      	} 
#      	if ($line =~ /(\d+).+available for\s+([\w:]+)/) {
# 	  @tmp= reverse split /:/, $2;	  
# 	  $minutes=$tmp[1] + 60*$tmp[2] + 24*60*$tmp[3];
# 	#  print  @tmp,"\t", $minutes, "\n";	  
#      	  $unix_mappings{$_} .= " ".$1.":".$minutes;
#      	#  print "$1 available for $2\n";
#      	}
#      	if ($line =~ /(\d+).+available with no timelimit/) {
#      	  $unix_mappings{$_}.= " ".$1;
#      	#  print "$1 procs with no limit\n";
#      	}	
#      }
#      close  showbfoutput; 
#    }    

   #start temporary solution to speed up the showbf, checks only a single user   	  
   open SHOWBFOUTPUT,	"$showbf_command  |" or die "Error in executing showbf";  
   while (my $line= <SHOWBFOUTPUT>) {						  
      if ($line =~ /^partition/) {						  
   	last;									  
      }   									  
      if ($line =~ /no procs available/) {					  
   	$maui_freecpus= " 0";      						  
   	last; 									  
      } 									  
      if ($line =~ /(\d+).+available for\s+([\w:]+)/) { 			  
   	@tmp= reverse split /:/, $2;	
   	$minutes=$tmp[1] + 60*$tmp[2] + 24*60*$tmp[3];  		
   	$maui_freecpus .= " ".$1.":".$minutes;	      			  
      } 									  
      if ($line =~ /(\d+).+available with no timelimit/) {			  
   	$maui_freecpus.= " ".$1;     					  
   	last; 
      }       									  
   }										  
   close  SHOWBFOUTPUT; 							  
   for ( keys %unix_mappings) {    						  
   	$unix_mappings{$_}=$maui_freecpus;					  
   }										  
   #end temporary solution							  
}
   

# The nordugrid-info-group=users entry
print "dn: $config{dn}\n";
print "objectclass: Mds\n";
print "objectclass: nordugrid-info-group\n";
print "nordugrid-info-group-name: users\n";
# Mds-validfrom/to
&mds_valid;


#
# create the user entries for those members of the hash
# which are PBS allowed
#

my $usernumber = 0;
foreach my $key (sort sort_by_cn keys %users) {

   if ( @acl_users and !(grep /$users{$key}/, @acl_users)) {
   #   print "we skip $users{$key} \n";
      next
   }
   

   #nordugrid-authuser-name= CN from the SN  + unique number 
   $key =~ m/\/CN=([^\/]+)(\/Email)?/;
   my $cn = $1;
   $usernumber++;
   print "dn: nordugrid-authuser-name=", $cn, "_", $usernumber, ", ", $config{dn}, "\n";
   print "objectclass: Mds\n";
   print "objectclass: nordugrid-authuser\n";
   print "nordugrid-authuser-name: ", $cn, "_", $usernumber, "\n";
   print "nordugrid-authuser-sn: $key\n";
   
 
   #nordugrid-authuser-freecpus
   if (lc($config{scheduling_policy}) eq "maui") {
      print "nordugrid-authuser-freecpus:", $unix_mappings{$users{$key}}, "\n";
   }
   else {   
      $systemlimit = $totalcpus - $usedcpus;
      if  ($max_running) {   
           $queuelimit =  $max_running - $totalrunning_in_the_queue;
      }    
      else  { $queuelimit =  $systemlimit; }
      if  ($max_userrun) {   
             $userlimit  =  $max_userrun - $user_jobs_running{$users{$key}};
      }
      else { $userlimit  =  $systemlimit; }    
   
      #print "SYSTEM:$systemlimit\n";
      #print "QUEUE:$queuelimit\n";;
      #print "USER:$userlimit\n";   
         
      @sorted_limits=sort {$a<=>$b}($systemlimit, $queuelimit, $userlimit);
  
      if ($sorted_limits[0] >0 ) {
         print "nordugrid-authuser-freecpus: $sorted_limits[0]$max_cputime\n";
      }
      else {
         print "nordugrid-authuser-freecpus: 0\n";
      }
   }
      
       
   #nordugrid-authuser-queuelength 
   if ($user_jobs_queued{$users{$key}}) {
       $queuelength=$user_jobs_queued{$users{$key}};
   }
   else { $queuelength=0};
   print "nordugrid-authuser-queuelength: ",$queuelength , "\n";   
   
   #nordugrid-authuser-diskspace
   my $gridarea = $config{sessiondir};
   my $diskspace;
   if ( -d "$gridarea") {
     # check if on afs
     if ($gridarea =~ /\/afs\//) {
   	   $diskspace =`fs listquota $gridarea 2>/dev/null`;
   	   if ($diskspace) {
   	     $diskspace =~ /\n\S+\s+(\d+)\s+(\d+)\s+\d+%\s+\d+%/;
   	     $diskspace = int (($1 - $2)/1024);
   	   } else {
   	     $diskspace = 0;
   	   }
     # "ordinary" disk
     } else {
   	   $diskspace =`df $gridarea 2>/dev/null`;
   	   if ($diskspace) {
   	     $diskspace =~ /\n\S+\s+\d+\s+\d+\s+(\d+)\s+\d+/;
   	     $diskspace=int $1/1024;
   	   } else {
   	     $diskspace = 0;
   	   }
     }
   } else {
     $diskspace=0;
   }   
   print "nordugrid-authuser-diskspace: $diskspace\n";

   #Mds-validfrom/to
   &mds_valid;
}


#sort by CN
sub sort_by_cn{
   $a =~ m/\/CN=([^\/]+)(\/Email)?/;
   $cn_a=$1;
   $b =~ m/\/CN=([^\/]+)(\/Email)?/;
   $cn_b=$1;   
   $cn_a cmp $cn_b;
}   

sub mds_valid {
# Mds-validfrom/to
  my $ttl = $config{ttl};
  my $seconds=time;
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($seconds);
  print "Mds-validfrom: ";
  printf "%4d%02d%02d%02d%02d%02d%1s",
        $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
        gmtime ($seconds+$ttl);
  print "Mds-validto: ";
  printf "%4d%02d%02d%02d%02d%02d%1s",
        $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
  print "\n";
}
